@@ -20,8 +20,8 @@
 *       cc -o whob whois.c -DSTANDALONE
 *
 *   
+*   Portions (c) Victor Oppleman (lft@oppleman.com)
 *   Portions (c) 2011 Markus Gothe <nietzsche@lysator.liu.se>
-*   Portions (c) 2007 Victor Oppleman (lft@oppleman.com)
 *   Portions (c) 2002 Ugen Antsilevitch (ugen@xonix.com)
 *
 */
@@ -47,10 +47,6 @@
 
 /* OPTIONS and variable initialization */
 
-static const char *version = "3.7";             /* set version string for library and client */
-#ifndef STANDALONE
-static const char appname[] = "pWhoIs C Library";   /* set application name */
-#endif
 static char pwhois_server[] = "whois.pwhois.org";
 static char radb_server[] = "whois.ra.net";
 static char cymru_server[] = "whois.cymru.com";
@@ -60,7 +56,8 @@ static char ripe_server[] = "whois.ripe.net";
 static char ripe_ris_server[] = "riswhois.ripe.net";
 
 #ifdef STANDALONE
-static const char appname[] = "WhoB";           /* set application name */
+const char *version = "3.71";                    /* set version string for library and client */
+const char *appname = "WhoB";            	/* set application name */
 static int use_cymru = 0;                       /* Don't use Cymru by default */
 static int use_riswhois = 0;                    /* Don't use RIPE NCC RIS by default */
 static int display_orgname = 1;                 /* Display orgname by default */
@@ -77,22 +74,13 @@ static int read_from_file = 0;                  /* Don't read input from file by
 static int riswhoisfromfile = 0;                /* Don't use riswhois by default for readfromfile */
 static int cymrufromfile = 0;                   /* Don't use Cymru by default for readfromfile */
 static int use_gigo = 1;                        /* Use GIGO feature by default */
+static int use_stdin = 0;                       /* Don't use STDIN for bulk file input by default */
 static const unsigned int max_hostname_input = 200;      /* Maximum length of input from user */
-static const int max_lines = 1000;              /* Maximum lines to read from bulk file per query */
+static const int max_lines = 2500;              /* Maximum lines to read from bulk file per query */
 static const int line_size = 256;               /* Maximum line length */
 static char hostname[256];
 #endif
 
-/* Optimize DNS resolution */
-/*
-    #include <resolv.h>
-    extern struct __res_state _res;
-
-    res_init();
-    _res.retrans = 1;
-    _res.retry = 1;
-*/
-
 /* END of OPTIONS and variable initialization */
 #if defined(WIN32) || defined(_WIN32)
 char *
@@ -768,7 +756,7 @@ w_lookup_all_riswhois_bulk(whois_session_params * wsess, struct ip_list_array *i
                 if(wsess->logprintfCookie)
                     lft_printf(wsess->logprintfCookie,"Skipping additional object for same query.\n");
                 else
-                    printf("Skipping additional object for same query.\n");
+                    fprintf(stderr,"Skipping additional object for same query.\n");
             }
         } else
         if ((value = match_prefix("descr:", responses[i].ptr))) {
@@ -981,7 +969,7 @@ w_lookup_as_cymru_bulk(whois_session_params * wsess, struct ip_list_array *iplis
                     if(wsess->logprintfCookie)
                         lft_printf(wsess->logprintfCookie,"Parse error at \'%c\': non-numeric value at position %d of %s).\n",value[i],i,value);
                     else
-                        printf("Parse error at \'%c\': non-numeric value at position %d of %s).\n",value[i],i,value);
+                        fprintf(stderr,"Parse error at \'%c\': non-numeric value at position %d of %s).\n",value[i],i,value);
                 }
                 break;
             }
@@ -1188,14 +1176,17 @@ static int w_display_bulkfromfile_pwhois(whois_session_params * wsess, char *fil
     *whob_query = '\0';
     
     if (!filespec) {
-        fprintf(stderr,"You must specify a file to use for bulk query input.\n");
+        fprintf(stderr,"You must specify a file to use (or '-' for stdin) for bulk query input.\n");
         free(lines);
 	free(this_line);
 	free(whob_query);
         exit(EXIT_FAILURE);
     }  
     
-    bulkFile = fopen(filespec, "r");
+    if (!strncmp(filespec,"-",1) || use_stdin == 1) {
+        bulkFile = stdin;
+    } else 
+        bulkFile = fopen(filespec, "r");
     
     if (!bulkFile) {
         fprintf(stderr,"%s: Unable to open \'%s\' for reading.\n",appname,filespec);
@@ -1293,14 +1284,17 @@ static int w_display_bulkfromfile_riswhois(whois_session_params * wsess, char *f
     *whob_query = '\0';
     
     if (!filespec) {
-        fprintf(stderr,"You must specify a file to use for bulk query input.\n");
+        fprintf(stderr,"You must specify a file to use (or '-' for stdin) for bulk query input.\n");
         free(lines);
-	free(this_line);
-	free(whob_query);
+        free(this_line);
+        free(whob_query);
         exit(EXIT_FAILURE);
-    }  
-    
-    bulkFile = fopen(filespec, "r");
+    }
+
+    if (!strncmp(filespec,"-",1) || use_stdin == 1) {
+        bulkFile = stdin;
+    } else
+        bulkFile = fopen(filespec, "r");
     
     if (!bulkFile) {
         fprintf(stderr,"%s: Unable to open \'%s\' for reading.\n",appname,filespec);
@@ -1389,14 +1383,17 @@ static int w_display_bulkfromfile_cymru(whois_session_params * wsess, char *file
     *whob_query = '\0';
 
     if (!filespec) {
-        fprintf(stderr,"You must specify a file to use for bulk query input.\n");
+        fprintf(stderr,"You must specify a file to use (or '-' for stdin) for bulk query input.\n");
         free(lines);
-	free(this_line);
-	free(whob_query);
+        free(this_line);
+        free(whob_query);
         exit(EXIT_FAILURE);
-    }  
-    
-    bulkFile = fopen(filespec, "r");
+    }
+
+    if (!strncmp(filespec,"-",1) || use_stdin == 1) {
+        bulkFile = stdin;
+    } else
+        bulkFile = fopen(filespec, "r");
     
     if (!bulkFile) {
         fprintf(stderr,"%s: Unable to open \'%s\' for reading.\n",appname,filespec);
@@ -1479,7 +1476,7 @@ static int w_display_pwhois_version(whois_session_params * wsess)
         serv = pwhois_server;
     
     if (wsess->w_noisy)
-        printf("Querying '%s' for version/status.\n",serv);
+        fprintf(stderr,"Querying '%s' for version/status.\n",serv);
     
     reply = w_ask(serv, "version", NULL);
     if (!reply) {
@@ -1505,7 +1502,7 @@ static int w_display_pwhois_gigo(whois_session_params * wsess, char *user_query)
         serv = pwhois_server;
     
     if (wsess->w_noisy)
-        printf("Querying '%s' for: '%s'\n",serv,user_query);
+        fprintf(stderr,"Querying '%s' for: '%s'\n",serv,user_query);
     
     reply = w_ask(serv, user_query, NULL);
     if (!reply) {
@@ -1605,7 +1602,7 @@ w_lookup_all_pwhois_bulk(whois_session_params * wsess, struct ip_list_array *ipl
                         if(wsess->logprintfCookie)
                             lft_printf(wsess->logprintfCookie,"Parse error at \'%c\': non-numeric value at position %d of %s).\n",value[i],i,value);
                         else
-                            printf("Parse error at \'%c\': non-numeric value at position %d of %s).\n",value[i],i,value);
+                            fprintf(stderr,"Parse error at \'%c\': non-numeric value at position %d of %s).\n",value[i],i,value);
                     }
                     break;
                 }
@@ -1721,7 +1718,7 @@ w_lookup_all_pwhois_bulk_ext(whois_session_params * wsess, struct ext_ip_list_ar
                         if(wsess->logprintfCookie)
                             lft_printf(wsess->logprintfCookie,"Parse error at \'%c\': non-numeric value at position %d of %s).\n",value[i],i,value);
                         else
-                            printf("Parse error at \'%c\': non-numeric value at position %d of %s).\n",value[i],i,value);
+                            fprintf(stderr,"Parse error at \'%c\': non-numeric value at position %d of %s).\n",value[i],i,value);
                     }
                     break;
                 }
@@ -1762,7 +1759,7 @@ w_lookup_all_pwhois_bulk_ext(whois_session_params * wsess, struct ext_ip_list_ar
                         if(wsess->logprintfCookie)
                             lft_printf(wsess->logprintfCookie,"Parse error at \'%c\': can't parse value at position %d of %s).\n",value[i],i,value);
                         else
-                            printf("Parse error at \'%c\': can't parse value at position %d of %s).\n",value[i],i,value);
+                            fprintf(stderr,"Parse error at \'%c\': can't parse value at position %d of %s).\n",value[i],i,value);
                     }
                     break;
                 }
@@ -1781,7 +1778,7 @@ w_lookup_all_pwhois_bulk_ext(whois_session_params * wsess, struct ext_ip_list_ar
                         if(wsess->logprintfCookie)
                             lft_printf(wsess->logprintfCookie,"Parse error at \'%c\': can't parse value at position %d of %s).\n",value[i],i,value);
                         else
-                            printf("Parse error at \'%c\': can't parse value at position %d of %s).\n",value[i],i,value);
+                           fprintf(stderr,"Parse error at \'%c\': can't parse value at position %d of %s).\n",value[i],i,value);
                     }
                     break;
                 }
@@ -2258,7 +2255,7 @@ usage (char *prog)
              "  -o         Disable display of the organization\'s name on file at the registrar\n"
              "\nAdvanced Options:\n"             
              "  -h host    Specify your own pWhoIs-compatible server to query\n"
-             "  -f file    Read from the specified file as bulk input to pWhoIs\n"
+             "  -f file    Read from 'file' (or '-' as stdin) as bulk input to pWhoIs\n"
              "  -c         Use Cymru\'s whois server instead of pWhoIs\n"
              "  -r         Use RIPE NCC\'s RIS whois server instead of pWhoIs\n"
              "\nPrefix WhoIs Advanced Options:\n"
@@ -2430,17 +2427,43 @@ int main(int ac, char **av)
             default:
                 usage (av[0]);
         }
-            
-            if (((ac - optind) < 1) && (show_server_status != 1))
-                usage (av[0]);
-    
+                    
+    /*  Quickly check stdin for input to avoid David having to type   */
+    /*  '-f -' to pipe into to whob, even if it has args :-)          */
+        fd_set rfds;
+        struct timeval timev;
+        int selretval;
+     
+        /* Watch stdin (fd 0) briefly for input. */
+        FD_ZERO(&rfds);
+        FD_SET(0, &rfds);
+        
+        timev.tv_sec = 0;
+        timev.tv_usec = 100;
+     
+        selretval = select(1, &rfds, NULL, NULL, &timev);
+     
+        if (selretval) {
+            /* There's input in them there hills, treat it as the input file */
+            use_gigo = 0;
+            read_from_file = 1;
+            use_stdin = 1;
+        } 
+        /*
+          else
+            if (wsess->w_noisy) fprintf(stderr,"No input found on stdin.\n");
+        */
+        
+    if (((ac - optind) < 1) && (show_server_status != 1) && (!read_from_file))
+        usage (av[0]);
+        
     /* Show the verbose startup information if verbosity is enabled/requested */
     show_startup_msgs(wsess);
-    
+        
     if (show_server_status == 1) {
         w_display_pwhois_version(wsess);
         exit(EXIT_FAILURE);
-    } else { 
+    } else if (use_stdin < 1) { 
         if (strlen(av[optind]) > max_hostname_input) {
             fprintf(stderr,"Sorry, the host name you supplied was unreasonably long.\n");
             exit(EXIT_FAILURE);
@@ -2449,7 +2472,7 @@ int main(int ac, char **av)
             optind++;
         }
     }
-    
+            
     if (read_from_file) {
         if (riswhoisfromfile) 
             w_display_bulkfromfile_riswhois(wsess, hostname);
@@ -2467,7 +2490,7 @@ int main(int ac, char **av)
     
     if ((show_routes_byasn || show_contacts_byasn || show_networks_byasn) && (strlen(hostname) <= 10) && atoi(hostname)) {
         user_asn = atoi(hostname);
-        if (wsess->w_noisy > 1) printf("Using user-supplied ASN %d for lookup.\n",user_asn);
+        if (wsess->w_noisy > 1) fprintf(stderr,"Using user-supplied ASN %d for lookup.\n",user_asn);
     } else if (show_routes_byprefix == 1) {
         printf("Displaying all routes for prefix %s.\n",hostname);
     } else {
